@@ -8,9 +8,6 @@ def load_setup_py_data(setup_file, from_recipe_dir=False, recipe_dir=None, work_
     _setuptools_data = {}
     log = logging.getLogger(__name__)
 
-    def setup(**kw):
-        _setuptools_data.update(kw)
-
     import setuptools
     import distutils.core
 
@@ -21,7 +18,10 @@ def load_setup_py_data(setup_file, from_recipe_dir=False, recipe_dir=None, work_
         setup_file = os.path.abspath(os.path.join(recipe_dir, setup_file))
     elif os.path.exists(work_dir):
         cd_to_work = True
-        cwd = os.getcwd()
+        try:
+            cwd = os.getcwd()
+        except OSError:
+            cwd = recipe_dir or work_dir
         os.chdir(work_dir)
         if not os.path.isabs(setup_file):
             setup_file = os.path.join(work_dir, setup_file)
@@ -37,6 +37,25 @@ def load_setup_py_data(setup_file, from_recipe_dir=False, recipe_dir=None, work_
         else:
             raise RuntimeError(message)
 
+    setup_cfg_data = {}
+    try:
+        from setuptools.config import read_configuration
+    except ImportError:
+        pass  # setuptools <30.3.0 cannot read metadata / options from 'setup.cfg'
+    else:
+        setup_cfg = os.path.join(os.path.dirname(setup_file), 'setup.cfg')
+        if os.path.isfile(setup_cfg):
+            # read_configuration returns a dict of dicts. Each dict (keys: 'metadata',
+            # 'options'), if present, provides keyword arguments for the setup function.
+            for kwargs in read_configuration(setup_cfg).values():
+                # explicit arguments to setup.cfg take priority over values in setup.py
+                setup_cfg_data.update(kwargs)
+
+    def setup(**kw):
+        _setuptools_data.update(kw)
+        # values in setup.cfg take priority over explicit arguments to setup.py
+        _setuptools_data.update(setup_cfg_data)
+
     # Patch setuptools, distutils
     setuptools_setup = setuptools.setup
     distutils_setup = distutils.core.setup
@@ -61,8 +80,9 @@ def load_setup_py_data(setup_file, from_recipe_dir=False, recipe_dir=None, work_
         '__file__': setup_file,
     }
     if os.path.isfile(setup_file):
-        code = compile(open(setup_file).read(), setup_file, 'exec', dont_inherit=1)
-        exec(code, ns, ns)
+        with open(setup_file) as f:
+            code = compile(f.read(), setup_file, 'exec', dont_inherit=1)
+            exec(code, ns, ns)
     else:
         if not permit_undefined_jinja:
             raise TypeError('{} is not a file that can be read'.format(setup_file))
@@ -73,6 +93,7 @@ def load_setup_py_data(setup_file, from_recipe_dir=False, recipe_dir=None, work_
     setuptools.setup = setuptools_setup
     if numpy_setup:
         numpy.distutils.core.setup = numpy_setup
+
     if cd_to_work:
         os.chdir(cwd)
     # remove our workdir from sys.path
