@@ -18,6 +18,8 @@ import sys
 import tarfile
 import tempfile
 
+from conda_build.utils import filter_info_files
+
 
 def retrieve_c_extensions(file_path, show_imports=False):
     """Check tarfile for compiled C files with '.pyd' or '.so' suffixes.
@@ -56,7 +58,13 @@ def retrieve_package_platform(file_path):
         index = json.loads(tar.extractfile('info/index.json').read().decode('utf-8'))
 
     platform = index['platform']
-    architecture = '64' if index['arch'] == 'x86_64' else '32'
+
+    if index.get('arch') == 'x86_64':
+        architecture = '64'
+    elif index.get('arch') == 'x86':
+        architecture = '32'
+    else:
+        architecture = index.get('arch')
 
     if platform.startswith('linux') or platform.startswith('osx'):
         return ('unix', platform, architecture)
@@ -169,7 +177,14 @@ def update_index_file(temp_dir, target_platform, dependencies, verbose):
         index = json.load(file)
 
     platform, architecture = target_platform.split('-')
-    source_architecture = '64' if index['arch'] == 'x86_64' else '32'
+    other_platforms = ['linux-ppc64le', 'linux-armv6l', 'linux-armv7l', 'linux-aarch64']
+
+    if target_platform in other_platforms:
+        source_architecture = architecture
+    elif index.get('arch') == 'x86_64':
+        source_architecture = '64'
+    else:
+        source_architecture = '32'
 
     if verbose:
         print('Updating platform from {} to {}' .format(index['platform'], platform))
@@ -178,13 +193,19 @@ def update_index_file(temp_dir, target_platform, dependencies, verbose):
 
     index['platform'] = platform
     index['subdir'] = target_platform
-    index['arch'] = 'x86_64' if architecture == '64' else 'x86'
+
+    if architecture == '64':
+        index['arch'] = 'x86_64'
+    elif architecture == '32':
+        index['arch'] = 'x86'
+    else:
+        index['arch'] = architecture
 
     if dependencies:
         index['depends'] = update_dependencies(dependencies, index['depends'])
 
     with open(index_file, 'w') as file:
-        json.dump(index, file)
+        json.dump(index, file, indent=2)
 
     return index_file
 
@@ -234,7 +255,8 @@ def update_lib_contents(lib_directory, temp_dir, target_platform, file_path):
         try:
             for lib_file in glob.iglob('{}/python*/**' .format(lib_directory)):
                 if 'site-packages' in lib_file:
-                    new_site_packages_path = os.path.join(temp_dir, 'lib/site-packages')
+                    new_site_packages_path = os.path.join(
+                        temp_dir, os.path.join('lib', 'site-packages'))
                     os.renames(lib_file, new_site_packages_path)
                 else:
                     if retrieve_python_version(lib_file) is not None:
@@ -251,15 +273,14 @@ def update_lib_contents(lib_directory, temp_dir, target_platform, file_path):
         os.rename(os.path.join(temp_dir, 'lib'), os.path.join(temp_dir, 'Lib'))
 
     elif target_platform == 'unix':
+        for lib_file in glob.iglob('{}/**' .format(lib_directory)):
+            python_version = retrieve_python_version(file_path)
+            new_lib_file = re.sub('Lib', os.path.join('lib', python_version), lib_file)
+            os.renames(lib_file, new_lib_file)
+
         try:
-            for lib_file in glob.iglob('{}/**' .format(lib_directory)):
-                python_version = retrieve_python_version(file_path)
-                new_lib_file = re.sub('Lib', os.path.join('lib', python_version), lib_file)
-                os.renames(lib_file, new_lib_file)
-
             os.rename(os.path.join(temp_dir, 'Lib'), os.path.join(temp_dir, 'lib'))
-
-        except OSError:
+        except:
             pass
 
 
@@ -347,7 +368,7 @@ def update_paths_file(temp_dir, target_platform):
                     paths['paths'].remove(path)
 
         with open(paths_file, 'w') as file:
-            json.dump(paths, file)
+            json.dump(paths, file, indent=2)
 
 
 def retrieve_executable_name(executable):
@@ -499,16 +520,18 @@ def update_files_file(temp_dir, verbose):
     """
     files_file = os.path.join(temp_dir, 'info/files')
 
-    with open(files_file, 'w+') as files:
+    with open(files_file, 'w') as files:
+        file_paths = []
         for dirpath, dirnames, filenames in os.walk(temp_dir):
-            for filename in filenames:
-                package_file_path = os.path.join(
-                    dirpath, filename).replace(temp_dir, '').lstrip(os.sep)
-                if not package_file_path.startswith('info'):
-                    files.write(package_file_path + '\n')
+            relative_dir = os.path.relpath(dirpath, temp_dir)
+            filenames = [os.path.join(relative_dir, f) for f in filenames]
+            for filename in filter_info_files(filenames, ''):
+                file_paths.append(filename)
+                if verbose:
+                    print('Updating {}' .format(filename))
 
-                    if verbose:
-                        print('Updating {}' .format(package_file_path))
+        for file_path in sorted(file_paths):
+            files.write(file_path + '\n')
 
 
 def create_target_archive(file_path, temp_dir, platform, output_dir):
@@ -530,10 +553,10 @@ def create_target_archive(file_path, temp_dir, platform, output_dir):
 
     with tarfile.open(destination, 'w:bz2') as target:
         for dirpath, dirnames, filenames in os.walk(temp_dir):
+            relative_dir = os.path.relpath(dirpath, temp_dir)
+            filenames = [os.path.join(relative_dir, f) for f in filenames]
             for filename in filenames:
-                destination_file_path = os.path.join(
-                    dirpath, filename).replace(temp_dir, '').lstrip(os.sep)
-                target.add(os.path.join(dirpath, filename), arcname=destination_file_path)
+                target.add(os.path.join(temp_dir, filename), arcname=filename)
 
 
 def convert_between_unix_platforms(file_path, output_dir, platform, dependencies, verbose):
@@ -700,7 +723,9 @@ def conda_convert(file_path, output_dir=".", show_imports=False, platforms=None,
     source_platform_architecture = '{}-{}' .format(source_platform, architecture)
 
     if 'all' in platforms:
-        platforms = ['osx-64', 'linux-32', 'linux-64', 'win-32', 'win-64']
+        platforms = ['osx-64', 'linux-32', 'linux-64',
+                     'linux-ppc64le', 'linux-armv6l', 'linux-armv7l', 'linux-aarch64',
+                     'win-32', 'win-64']
 
     for platform in platforms:
 
