@@ -2,10 +2,10 @@ from __future__ import absolute_import, division, print_function
 
 import io
 import locale
-import logging
 import os
-from os.path import join, isdir, isfile, abspath, basename, exists, normpath
+from os.path import join, isdir, isfile, abspath, basename, exists, normpath, expanduser
 import re
+import shutil
 from subprocess import CalledProcessError
 import sys
 import time
@@ -16,14 +16,9 @@ from .conda_interface import hashsum_file
 from conda_build.os_utils import external
 from conda_build.conda_interface import url_path, CondaHTTPError
 from conda_build.utils import (tar_xf, unzip, safe_print_unicode, copy_into, on_win, ensure_list,
-                               check_output_env, check_call_env, convert_path_for_cygwin_or_msys2)
+                               check_output_env, check_call_env, convert_path_for_cygwin_or_msys2,
+                               get_logger, rm_rf, LoggingContext)
 
-# legacy exports for conda
-from .config import Config as _Config
-SRC_CACHE = _Config().src_cache
-GIT_CACHE = _Config().git_cache
-SVN_CACHE = _Config().svn_cache
-HG_CACHE = _Config().hg_cache
 
 if on_win:
     from conda_build.utils import convert_unix_path_to_win
@@ -34,77 +29,144 @@ else:
     from urlparse import urljoin
 
 git_submod_re = re.compile(r'(?:.+)\.(.+)\.(?:.+)\s(.+)')
+ext_re = re.compile(r"(.*?)(\.(?:tar\.)?[^.]+)$")
 
 
-def download_to_cache(metadata, config):
+def append_hash_to_fn(fn, hash_value):
+    return ext_re.sub(r"\1_{}\2".format(hash_value[:10]), fn)
+
+
+def download_to_cache(cache_folder, recipe_path, source_dict):
     ''' Download a source to the local cache. '''
-    print('Source cache directory is: %s' % config.src_cache)
-    if not isdir(config.src_cache):
-        os.makedirs(config.src_cache)
-    meta = metadata.get_section('source')
+    print('Source cache directory is: %s' % cache_folder)
+    if not isdir(cache_folder):
+        os.makedirs(cache_folder)
 
-    fn = meta['fn'] if 'fn' in meta else basename(meta['url'])
-    path = join(config.src_cache, fn)
+    unhashed_fn = fn = source_dict['fn'] if 'fn' in source_dict else basename(source_dict['url'])
+    hash_added = False
+    for hash_type in ('md5', 'sha1', 'sha256'):
+        if hash_type in source_dict:
+            fn = append_hash_to_fn(fn, source_dict[hash_type])
+            hash_added = True
+            break
+    else:
+        log = get_logger(__name__)
+        log.warn("No hash (md5, sha1, sha256) provided.  Source download forced.  "
+                 "Add hash to recipe to use source cache.")
+    path = join(cache_folder, fn)
     if isfile(path):
         print('Found source in cache: %s' % fn)
     else:
         print('Downloading source to cache: %s' % fn)
-        if not isinstance(meta['url'], list):
-            meta['url'] = [meta['url']]
+        if not isinstance(source_dict['url'], list):
+            source_dict['url'] = [source_dict['url']]
 
-        for url in meta['url']:
+        for url in source_dict['url']:
             if "://" not in url:
+                if url.startswith('~'):
+                    url = expanduser(url)
                 if not os.path.isabs(url):
-                    url = os.path.normpath(os.path.join(metadata.path, url))
+                    url = os.path.normpath(os.path.join(recipe_path, url))
                 url = url_path(url)
+            else:
+                if url.startswith('file:///~'):
+                    url = 'file:///' + expanduser(url[8:]).replace('\\', '/')
             try:
                 print("Downloading %s" % url)
-                download(url, path)
+                with LoggingContext():
+                    download(url, path)
             except CondaHTTPError as e:
                 print("Error: %s" % str(e).strip(), file=sys.stderr)
+                rm_rf(path)
             except RuntimeError as e:
                 print("Error: %s" % str(e).strip(), file=sys.stderr)
+                rm_rf(path)
             else:
                 print("Success")
                 break
         else:  # no break
-            raise RuntimeError("Could not download %s" % fn)
+            rm_rf(path)
+            raise RuntimeError("Could not download %s" % url)
 
-    for tp in 'md5', 'sha1', 'sha256':
-        if meta.get(tp) and hashsum_file(path, tp) != meta[tp]:
-            raise RuntimeError("%s mismatch: '%s' != '%s'" %
-                               (tp.upper(), hashsum_file(path, tp), meta[tp]))
+    hashed = None
+    for tp in ('md5', 'sha1', 'sha256'):
+        if tp in source_dict:
+            expected_hash = source_dict[tp]
+            hashed = hashsum_file(path, tp)
+            if expected_hash != hashed:
+                rm_rf(path)
+                raise RuntimeError("%s mismatch: '%s' != '%s'" %
+                           (tp.upper(), hashed, expected_hash))
+            break
 
-    return path
+    # this is really a fallback.  If people don't provide the hash, we still need to prevent
+    #    collisions in our source cache, but the end user will get no benefirt from the cache.
+    if not hash_added:
+        if not hashed:
+            hashed = hashsum_file(path, 'sha256')
+        dest_path = append_hash_to_fn(path, hashed)
+        if not os.path.isfile(dest_path):
+            os.rename(path, dest_path)
+        path = dest_path
+
+    return path, unhashed_fn
 
 
-def unpack(meta, config):
+def hoist_single_extracted_folder(nested_folder):
+    """Moves all files/folders one level up.
+
+    This is for when your archive extracts into its own folder, so that we don't need to
+    know exactly what that folder is called."""
+    flist = os.listdir(nested_folder)
+    parent = os.path.dirname(nested_folder)
+    nested_folders_to_remove = [nested_folder]
+    for thing in flist:
+        if not os.path.isdir(os.path.join(parent, thing)):
+            shutil.move(os.path.join(nested_folder, thing), os.path.join(parent, thing))
+        else:
+            copy_into(os.path.join(nested_folder, thing), os.path.join(parent, thing))
+            nested_folders_to_remove.append(os.path.join(nested_folder, thing))
+    for folder in nested_folders_to_remove:
+        rm_rf(folder)
+
+
+def unpack(source_dict, src_dir, cache_folder, recipe_path, croot, verbose=False,
+           timeout=90, locking=True):
     ''' Uncompress a downloaded source. '''
-    src_path = download_to_cache(meta, config)
+    src_path, unhashed_fn = download_to_cache(cache_folder, recipe_path, source_dict)
 
-    if not isdir(config.work_dir):
-        os.makedirs(config.work_dir)
-    if config.verbose:
+    if not isdir(src_dir):
+        os.makedirs(src_dir)
+    if verbose:
         print("Extracting download")
-    if src_path.lower().endswith(('.tar.gz', '.tar.bz2', '.tgz', '.tar.xz',
-            '.tar', 'tar.z')):
-        tar_xf(src_path, config.work_dir)
-    elif src_path.lower().endswith('.zip'):
-        unzip(src_path, config.work_dir)
-    elif src_path.lower().endswith('.whl'):
-        # copy wheel itself *and* unpack it
-        # This allows test_files or about.license_file to locate files in the wheel,
-        # as well as `pip install name-version.whl` as install command
-        unzip(src_path, config.work_dir)
-        copy_into(src_path, config.work_dir, config.timeout, locking=config.locking)
-    else:
-        # In this case, the build script will need to deal with unpacking the source
-        print("Warning: Unrecognized source format. Source file will be copied to the SRC_DIR")
-        copy_into(src_path, config.work_dir, config.timeout, locking=config.locking)
+    with TemporaryDirectory(dir=croot) as tmpdir:
+        unhashed_dest = os.path.join(tmpdir, unhashed_fn)
+        if src_path.lower().endswith(('.tar.gz', '.tar.bz2', '.tgz', '.tar.xz',
+                '.tar', 'tar.z')):
+            tar_xf(src_path, tmpdir)
+        elif src_path.lower().endswith('.zip'):
+            unzip(src_path, tmpdir)
+        elif src_path.lower().endswith('.whl'):
+            # copy wheel itself *and* unpack it
+            # This allows test_files or about.license_file to locate files in the wheel,
+            # as well as `pip install name-version.whl` as install command
+            unzip(src_path, tmpdir)
+            copy_into(src_path, unhashed_dest, timeout, locking=locking)
+        else:
+            # In this case, the build script will need to deal with unpacking the source
+            print("Warning: Unrecognized source format. Source file will be copied to the SRC_DIR")
+            copy_into(src_path, unhashed_dest, timeout, locking=locking)
+        flist = os.listdir(tmpdir)
+        folder = os.path.join(tmpdir, flist[0])
+        if len(flist) == 1 and os.path.isdir(folder):
+            hoist_single_extracted_folder(folder)
+        flist = os.listdir(tmpdir)
+        for f in flist:
+            shutil.move(os.path.join(tmpdir, f), os.path.join(src_dir, f))
 
 
-def git_mirror_checkout_recursive(git, mirror_dir, checkout_dir, git_url, config, git_ref=None,
-                                  git_depth=-1, is_top_level=True):
+def git_mirror_checkout_recursive(git, mirror_dir, checkout_dir, git_url, git_cache, git_ref=None,
+                                  git_depth=-1, is_top_level=True, verbose=True):
     """ Mirror (and checkout) a Git repository recursively.
 
         It's not possible to use `git submodule` on a bare
@@ -120,7 +182,7 @@ def git_mirror_checkout_recursive(git, mirror_dir, checkout_dir, git_url, config
         folders unless steps are taken to prevent that.
     """
 
-    if config.verbose:
+    if verbose:
         stdout = None
         stderr = None
     else:
@@ -128,14 +190,15 @@ def git_mirror_checkout_recursive(git, mirror_dir, checkout_dir, git_url, config
         stdout = FNULL
         stderr = FNULL
 
-    if not mirror_dir.startswith(config.git_cache + os.sep):
+    if not mirror_dir.startswith(git_cache + os.sep):
         sys.exit("Error: Attempting to mirror to %s which is outside of GIT_CACHE %s"
-                 % (mirror_dir, config.git_cache))
+                 % (mirror_dir, git_cache))
 
     # This is necessary for Cygwin git and m2-git, although it is fixed in newer MSYS2.
-    git_mirror_dir = convert_path_for_cygwin_or_msys2(git, mirror_dir)
-    git_checkout_dir = convert_path_for_cygwin_or_msys2(git, checkout_dir)
+    git_mirror_dir = convert_path_for_cygwin_or_msys2(git, mirror_dir).rstrip('/')
+    git_checkout_dir = convert_path_for_cygwin_or_msys2(git, checkout_dir).rstrip('/')
 
+    mirror_dir = mirror_dir.rstrip('/')
     if not isdir(os.path.dirname(mirror_dir)):
         os.makedirs(os.path.dirname(mirror_dir))
     if isdir(mirror_dir):
@@ -177,7 +240,7 @@ def git_mirror_checkout_recursive(git, mirror_dir, checkout_dir, git_url, config
         if git_url.startswith('.'):
             output = check_output_env([git, "rev-parse", checkout], stdout=stdout, stderr=stderr)
             checkout = output.decode('utf-8')
-        if config.verbose:
+        if verbose:
             print('checkout: %r' % checkout)
         if checkout:
             check_call_env([git, 'checkout', checkout],
@@ -201,41 +264,41 @@ def git_mirror_checkout_recursive(git, mirror_dir, checkout_dir, git_url, config
             submod_url = urljoin(git_url + '/', submod_rel_path)
             submod_mirror_dir = os.path.normpath(
                 os.path.join(mirror_dir, submod_rel_path))
-            if config.verbose:
+            if verbose:
                 print('Relative submodule %s found: url is %s, submod_mirror_dir is %s' % (
                       submod_name, submod_url, submod_mirror_dir))
             with TemporaryDirectory() as temp_checkout_dir:
                 git_mirror_checkout_recursive(git, submod_mirror_dir, temp_checkout_dir, submod_url,
-                                              config, git_ref, git_depth, False)
+                                              git_cache=git_cache, git_ref=git_ref,
+                                              git_depth=git_depth, is_top_level=False,
+                                              verbose=verbose)
 
     if is_top_level:
         # Now that all relative-URL-specified submodules are locally mirrored to
         # relatively the same place we can go ahead and checkout the submodules.
         check_call_env([git, 'submodule', 'update', '--init',
                     '--recursive'], cwd=checkout_dir, stdout=stdout, stderr=stderr)
-        git_info(config)
-    if not config.verbose:
+        git_info(checkout_dir, verbose=verbose)
+    if not verbose:
         FNULL.close()
 
 
-def git_source(metadata, config):
+def git_source(source_dict, git_cache, src_dir, recipe_path=None, verbose=True):
     ''' Download a source from a Git repo (or submodule, recursively) '''
-    if not isdir(config.git_cache):
-        os.makedirs(config.git_cache)
+    if not isdir(git_cache):
+        os.makedirs(git_cache)
 
     git = external.find_executable('git')
     if not git:
         sys.exit("Error: git is not installed in your root environment.")
 
-    meta = metadata.get_section('source')
-
-    git_url = meta['git_url']
-    git_depth = int(meta.get('git_depth', -1))
-    git_ref = meta.get('git_rev', 'HEAD')
+    git_url = source_dict['git_url']
+    git_depth = int(source_dict.get('git_depth', -1))
+    git_ref = source_dict.get('git_rev', 'HEAD')
 
     if git_url.startswith('.'):
         # It's a relative path from the conda recipe
-        git_url = abspath(normpath(os.path.join(metadata.path, git_url)))
+        git_url = abspath(normpath(os.path.join(recipe_path, git_url)))
         if sys.platform == 'win32':
             git_dn = git_url.replace(':', '_')
         else:
@@ -245,23 +308,24 @@ def git_source(metadata, config):
         if git_dn.startswith(os.sep):
             git_dn = git_dn[1:]
         git_dn = git_dn.replace(':', '_')
-    mirror_dir = join(config.git_cache, git_dn)
+    mirror_dir = join(git_cache, git_dn)
     git_mirror_checkout_recursive(
-        git, mirror_dir, config.work_dir, git_url, config, git_ref, git_depth, True)
+        git, mirror_dir, src_dir, git_url, git_cache=git_cache, git_ref=git_ref,
+        git_depth=git_depth, is_top_level=True, verbose=verbose)
     return git
 
 
-def git_info(config, fo=None):
+def git_info(src_dir, verbose=True, fo=None):
     ''' Print info about a Git repo. '''
-    assert isdir(config.work_dir)
+    assert isdir(src_dir)
 
     git = external.find_executable('git')
     if not git:
-        log = logging.getLogger(__name__)
+        log = get_logger(__name__)
         log.warn("git not installed in root environment.  Skipping recording of git info.")
         return
 
-    if config.verbose:
+    if verbose:
         stderr = None
     else:
         FNULL = open(os.devnull, 'w')
@@ -270,14 +334,14 @@ def git_info(config, fo=None):
     # Ensure to explicitly set GIT_DIR as some Linux machines will not
     # properly execute without it.
     env = os.environ.copy()
-    env['GIT_DIR'] = join(config.work_dir, '.git')
+    env['GIT_DIR'] = join(src_dir, '.git')
     env = {str(key): str(value) for key, value in env.items()}
     for cmd, check_error in [
             ('git log -n1', True),
             ('git describe --tags --dirty', False),
             ('git status', True)]:
         try:
-            stdout = check_output_env(cmd.split(), stderr=stderr, cwd=config.work_dir, env=env)
+            stdout = check_output_env(cmd.split(), stderr=stderr, cwd=src_dir, env=env)
         except CalledProcessError as e:
             if check_error:
                 raise Exception("git error: %s" % str(e))
@@ -289,17 +353,17 @@ def git_info(config, fo=None):
             stdout = stdout.decode(encoding, 'ignore')
         if fo:
             fo.write(u'==> %s <==\n' % cmd)
-            if config.verbose:
+            if verbose:
                 fo.write(stdout + u'\n')
         else:
-            if config.verbose:
+            if verbose:
                 print(u'==> %s <==\n' % cmd)
                 safe_print_unicode(stdout + u'\n')
 
 
-def hg_source(metadata, config):
+def hg_source(source_dict, src_dir, hg_cache, verbose):
     ''' Download a source from Mercurial repo. '''
-    if config.verbose:
+    if verbose:
         stdout = None
         stderr = None
     else:
@@ -307,39 +371,36 @@ def hg_source(metadata, config):
         stdout = FNULL
         stderr = FNULL
 
-    meta = metadata.get_section('source')
-
-    hg = external.find_executable('hg', config.build_prefix)
-    if not hg:
-        sys.exit('Error: hg not installed')
-    hg_url = meta['hg_url']
-    if not isdir(config.hg_cache):
-        os.makedirs(config.hg_cache)
+    hg_url = source_dict['hg_url']
+    if not isdir(hg_cache):
+        os.makedirs(hg_cache)
     hg_dn = hg_url.split(':')[-1].replace('/', '_')
-    cache_repo = join(config.hg_cache, hg_dn)
+    cache_repo = join(hg_cache, hg_dn)
     if isdir(cache_repo):
-        check_call_env([hg, 'pull'], cwd=cache_repo, stdout=stdout, stderr=stderr)
+        check_call_env(['hg', 'pull'], cwd=cache_repo, stdout=stdout, stderr=stderr)
     else:
-        check_call_env([hg, 'clone', hg_url, cache_repo], stdout=stdout, stderr=stderr)
+        check_call_env(['hg', 'clone', hg_url, cache_repo], stdout=stdout, stderr=stderr)
         assert isdir(cache_repo)
 
     # now clone in to work directory
-    update = meta.get('hg_tag') or 'tip'
-    if config.verbose:
+    update = source_dict.get('hg_tag') or 'tip'
+    if verbose:
         print('checkout: %r' % update)
 
-    check_call_env([hg, 'clone', cache_repo, config.work_dir], stdout=stdout, stderr=stderr)
-    check_call_env([hg, 'update', '-C', update], cwd=config.work_dir, stdout=stdout, stderr=stderr)
+    check_call_env(['hg', 'clone', cache_repo, src_dir], stdout=stdout,
+                   stderr=stderr)
+    check_call_env(['hg', 'update', '-C', update], cwd=src_dir, stdout=stdout,
+                   stderr=stderr)
 
-    if not config.verbose:
+    if not verbose:
         FNULL.close()
 
-    return config.work_dir
+    return src_dir
 
 
-def svn_source(metadata, config):
+def svn_source(source_dict, src_dir, svn_cache, verbose=True, timeout=90, locking=True):
     ''' Download a source from SVN repo. '''
-    if config.verbose:
+    if verbose:
         stdout = None
         stderr = None
     else:
@@ -347,40 +408,35 @@ def svn_source(metadata, config):
         stdout = FNULL
         stderr = FNULL
 
-    meta = metadata.get_section('source')
-
     def parse_bool(s):
         return str(s).lower().strip() in ('yes', 'true', '1', 'on')
 
-    svn = external.find_executable('svn', config.build_prefix)
-    if not svn:
-        sys.exit("Error: svn is not installed")
-    svn_url = meta['svn_url']
-    svn_revision = meta.get('svn_rev') or 'head'
-    svn_ignore_externals = parse_bool(meta.get('svn_ignore_externals') or 'no')
-    if not isdir(config.svn_cache):
-        os.makedirs(config.svn_cache)
+    svn_url = source_dict['svn_url']
+    svn_revision = source_dict.get('svn_rev') or 'head'
+    svn_ignore_externals = parse_bool(source_dict.get('svn_ignore_externals') or 'no')
+    if not isdir(svn_cache):
+        os.makedirs(svn_cache)
     svn_dn = svn_url.split(':', 1)[-1].replace('/', '_').replace(':', '_')
-    cache_repo = join(config.svn_cache, svn_dn)
+    cache_repo = join(svn_cache, svn_dn)
     if svn_ignore_externals:
         extra_args = ['--ignore-externals']
     else:
         extra_args = []
     if isdir(cache_repo):
-        check_call_env([svn, 'up', '-r', svn_revision] + extra_args, cwd=cache_repo,
+        check_call_env(['svn', 'up', '-r', svn_revision] + extra_args, cwd=cache_repo,
                        stdout=stdout, stderr=stderr)
     else:
-        check_call_env([svn, 'co', '-r', svn_revision] + extra_args + [svn_url, cache_repo],
+        check_call_env(['svn', 'co', '-r', svn_revision] + extra_args + [svn_url, cache_repo],
                        stdout=stdout, stderr=stderr)
         assert isdir(cache_repo)
 
     # now copy into work directory
-    copy_into(cache_repo, config.work_dir, config.timeout, symlinks=True, locking=config.locking)
+    copy_into(cache_repo, src_dir, timeout, symlinks=True, locking=locking)
 
-    if not config.verbose:
+    if not verbose:
         FNULL.close()
 
-    return config.work_dir
+    return src_dir
 
 
 def get_repository_info(recipe_path):
@@ -406,7 +462,7 @@ def get_repository_info(recipe_path):
                                              time.ctime(os.path.getmtime(
                                                  join(recipe_path, "meta.yaml"))))
     except CalledProcessError:
-        logging.getLogger(__name__).debug("Failed to checkout source in " + recipe_path)
+        get_logger(__name__).debug("Failed to checkout source in " + recipe_path)
         return "{}, last modified {}".format(recipe_path,
                                              time.ctime(os.path.getmtime(
                                                  join(recipe_path, "meta.yaml"))))
@@ -415,10 +471,20 @@ def get_repository_info(recipe_path):
 def _ensure_unix_line_endings(path):
     """Replace windows line endings with Unix.  Return path to modified file."""
     out_path = path + "_unix"
-    with open(path) as inputfile:
-        with open(out_path, "w") as outputfile:
+    with open(path, "rb") as inputfile:
+        with open(out_path, "wb") as outputfile:
             for line in inputfile:
-                outputfile.write(line.replace("\r\n", "\n"))
+                outputfile.write(line.replace(b"\r\n", b"\n"))
+    return out_path
+
+
+def _ensure_win_line_endings(path):
+    """Replace unix line endings with win.  Return path to modified file."""
+    out_path = path + "_win"
+    with open(path, "rb") as inputfile:
+        with open(out_path, "wb") as outputfile:
+            for line in inputfile:
+                outputfile.write(line.replace(b"\n", b"\r\n"))
     return out_path
 
 
@@ -483,6 +549,7 @@ def apply_patch(src_dir, path, config, git=None):
         git_env['GIT_COMMITTER_EMAIL'] = 'conda@conda-build.org'
         check_call_env([git, 'am', '--committer-date-is-author-date', path],
                        cwd=src_dir, stdout=None, env=git_env)
+        config.git_commits_since_tag += 1
     else:
         print('Applying patch: %r' % path)
         patch = external.find_executable('patch', config.build_prefix)
@@ -494,53 +561,100 @@ def apply_patch(src_dir, path, config, git=None):
             or conda, m2-patch (Windows),
         """ % (os.pathsep.join(external.dir_paths)))
         patch_strip_level = _guess_patch_strip_level(files, src_dir)
-        patch_args = ['-p%d' % patch_strip_level, '-i', path]
-        if sys.platform == 'win32':
-            patch_args[-1] = _ensure_unix_line_endings(path)
-        check_call_env([patch] + patch_args, cwd=src_dir)
-        if sys.platform == 'win32' and os.path.exists(patch_args[-1]):
-            os.remove(patch_args[-1])  # clean up .patch_unix file
+        patch_args = ['-p%d' % patch_strip_level, '--ignore-whitespace', '-i', path]
+
+        # line endings are a pain.
+        # https://unix.stackexchange.com/a/243748/34459
+
+        try:
+            log = get_logger(__name__)
+            log.info("Trying to apply patch as-is")
+            check_call_env([patch] + patch_args, cwd=src_dir)
+        except CalledProcessError:
+            if sys.platform == 'win32':
+                unix_ending_file = _ensure_unix_line_endings(path)
+                patch_args[-1] = unix_ending_file
+                try:
+                    log.info("Applying unmodified patch failed.  "
+                             "Convert to unix line endings and trying again.")
+                    check_call_env([patch] + patch_args, cwd=src_dir)
+                except:
+                    log.info("Applying unix patch failed.  "
+                             "Convert to CRLF line endings and trying again with --binary.")
+                    patch_args.insert(0, '--binary')
+                    win_ending_file = _ensure_win_line_endings(path)
+                    patch_args[-1] = win_ending_file
+                    try:
+                        check_call_env([patch] + patch_args, cwd=src_dir)
+                    except:
+                        raise
+                    finally:
+                        if os.path.exists(win_ending_file):
+                            os.remove(win_ending_file)  # clean up .patch_win file
+                finally:
+                    if os.path.exists(unix_ending_file):
+                        os.remove(unix_ending_file)  # clean up .patch_unix file
+            else:
+                raise
 
 
-def provide(metadata, config, patch=True):
+def provide(metadata):
     """
     given a recipe_dir:
       - download (if necessary)
       - unpack
       - apply patches (if any)
     """
-    if not os.path.isdir(config.build_folder):
-        os.makedirs(config.build_folder)
+    meta = metadata.get_section('source')
+    if not os.path.isdir(metadata.config.build_folder):
+        os.makedirs(metadata.config.build_folder)
     git = None
 
-    meta = metadata.get_section('source')
+    if hasattr(meta, 'keys'):
+        dicts = [meta]
+    else:
+        dicts = meta
 
-    if any(k in meta for k in ('fn', 'url')):
-        unpack(metadata, config=config)
-    elif 'git_url' in meta:
-        git = git_source(metadata, config=config)
-    # build to make sure we have a work directory with source in it.  We want to make sure that
-    #    whatever version that is does not interfere with the test we run next.
-    elif 'hg_url' in meta:
-        hg_source(metadata, config=config)
-    elif 'svn_url' in meta:
-        svn_source(metadata, config=config)
-    elif 'path' in meta:
-        path = normpath(abspath(join(metadata.path, metadata.get_value('source/path'))))
-        if config.verbose:
-            print("Copying %s to %s" % (path, config.work_dir))
-        # careful here: we set test path to be outside of conda-build root in setup.cfg.
-        #    If you don't do that, this is a recursive function
-        copy_into(path, config.work_dir, config.timeout, symlinks=True,
-                  locking=config.locking)
-    else:  # no source
-        if not isdir(config.work_dir):
-            os.makedirs(config.work_dir)
+    try:
+        for source_dict in dicts:
+            folder = source_dict.get('folder')
+            src_dir = (os.path.join(metadata.config.work_dir, folder) if folder else
+                    metadata.config.work_dir)
+            if any(k in source_dict for k in ('fn', 'url')):
+                unpack(source_dict, src_dir, metadata.config.src_cache, recipe_path=metadata.path,
+                    croot=metadata.config.croot, verbose=metadata.config.verbose,
+                    timeout=metadata.config.timeout, locking=metadata.config.locking)
+            elif 'git_url' in source_dict:
+                git = git_source(source_dict, metadata.config.git_cache, src_dir, metadata.path,
+                                verbose=metadata.config.verbose)
+            # build to make sure we have a work directory with source in it. We
+            #    want to make sure that whatever version that is does not
+            #    interfere with the test we run next.
+            elif 'hg_url' in source_dict:
+                hg_source(source_dict, src_dir, metadata.config.hg_cache,
+                        verbose=metadata.config.verbose)
+            elif 'svn_url' in source_dict:
+                svn_source(source_dict, src_dir, metadata.config.svn_cache,
+                        verbose=metadata.config.verbose, timeout=metadata.config.timeout,
+                        locking=metadata.config.locking)
+            elif 'path' in source_dict:
+                path = normpath(abspath(join(metadata.path, source_dict['path'])))
+                if metadata.config.verbose:
+                    print("Copying %s to %s" % (path, src_dir))
+                # careful here: we set test path to be outside of conda-build root in setup.cfg.
+                #    If you don't do that, this is a recursive function
+                copy_into(path, src_dir, metadata.config.timeout, symlinks=True,
+                        locking=metadata.config.locking, clobber=True)
+            else:  # no source
+                if not isdir(src_dir):
+                    os.makedirs(src_dir)
 
-    if patch:
-        src_dir = config.work_dir
-        patches = ensure_list(meta.get('patches', []))
-        for patch in patches:
-            apply_patch(src_dir, join(metadata.path, patch), config, git)
+            patches = ensure_list(source_dict.get('patches', []))
+            for patch in patches:
+                apply_patch(src_dir, join(metadata.path, patch), metadata.config, git)
 
-    return config.work_dir
+    except CalledProcessError:
+        os.rename(metadata.config.work_dir, metadata.config.work_dir + '_failed_provide')
+        raise
+
+    return metadata.config.work_dir
