@@ -11,10 +11,9 @@ import setuptools  # noqa
 from distutils.msvc9compiler import find_vcvarsall as distutils_find_vcvarsall
 from distutils.msvc9compiler import Reg, WINSDK_BASE
 
-from .conda_interface import bits
-
 from conda_build import environ
-from conda_build.utils import check_call_env, root_script_dir, path_prepended, copy_into
+from conda_build.utils import check_call_env, root_script_dir, path_prepended, copy_into, get_logger
+from conda_build.variants import set_language_env_vars, get_default_variant
 
 
 assert sys.platform == 'win32'
@@ -30,7 +29,7 @@ VS_VERSION_STRING = {
 }
 
 
-def fix_staged_scripts(scripts_dir):
+def fix_staged_scripts(scripts_dir, config):
     """
     Fixes scripts which have been installed unix-style to have a .bat
     helper
@@ -54,7 +53,7 @@ def fix_staged_scripts(scripts_dir):
             with open(join(scripts_dir, fn + '-script.py'), 'wb') as fo:
                 fo.write(f.read())
             # now create the .exe file
-            copy_into(join(dirname(__file__), 'cli-%d.exe' % bits),
+            copy_into(join(dirname(__file__), 'cli-%s.exe' % config.host_arch),
                             join(scripts_dir, fn + '.exe'))
 
         # remove the original script
@@ -86,7 +85,13 @@ def build_vcvarsall_vs_path(version):
 
 
 def msvc_env_cmd(bits, config, override=None):
-    arch_selector = 'x86' if bits == 32 else 'amd64'
+    log = get_logger(__name__)
+    log.warn("Using legacy MSVC compiler setup.  This will be removed in conda-build 4.0. "
+             "If this recipe does not use a compiler, this message is safe to ignore.  "
+             "Otherwise, use {{compiler('<language>')}} jinja2 in requirements/build.")
+    # this has been an int at times.  Make sure it's a string for consistency.
+    bits = str(bits)
+    arch_selector = 'x86' if bits == '32' else 'amd64'
 
     msvc_env_lines = []
 
@@ -105,10 +110,11 @@ def msvc_env_cmd(bits, config, override=None):
     msvc_env_lines.append('set MSSdk=1')
 
     if not version:
-        if config.PY3K and config.use_MSVC2015:
+        py_ver = config.variant.get('python', get_default_variant(config)['python'])
+        if int(py_ver[0]) >= 3:
+            if int(py_ver.split('.')[1]) < 5:
+                version = '10.0'
             version = '14.0'
-        elif config.PY3K:
-            version = '10.0'
         else:
             version = '9.0'
 
@@ -128,7 +134,7 @@ def msvc_env_cmd(bits, config, override=None):
     msvc_env_lines.append('set "VS_MAJOR={}"'.format(version.split('.')[0]))
     msvc_env_lines.append('set "VS_YEAR={}"'.format(VS_VERSION_STRING[version][-4:]))
     msvc_env_lines.append('set "CMAKE_GENERATOR={}"'.format(VS_VERSION_STRING[version] +
-                                                            {64: ' Win64', 32: ''}[bits]))
+                                                            {'64': ' Win64', '32': ''}[bits]))
     # tell msys2 to ignore path conversions for issue-causing windows-style flags in build
     #   See https://github.com/conda-forge/icu-feedstock/pull/5
     msvc_env_lines.append('set "MSYS2_ARG_CONV_EXCL=/AI;/AL;/OUT;/out"')
@@ -139,7 +145,7 @@ def msvc_env_cmd(bits, config, override=None):
                                             'installationfolder')
             WIN_SDK_71_BAT_PATH = os.path.join(WIN_SDK_71_PATH, 'Bin', 'SetEnv.cmd')
 
-            win_sdk_arch = '/Release /x86' if bits == 32 else '/Release /x64'
+            win_sdk_arch = '/Release /x86' if bits == '32' else '/Release /x64'
             win_sdk_cmd = build_vcvarsall_cmd(WIN_SDK_71_BAT_PATH, arch=win_sdk_arch)
 
             # There are two methods of building Python 3.3 and 3.4 extensions (both
@@ -173,7 +179,7 @@ def msvc_env_cmd(bits, config, override=None):
         except (KeyError, TypeError):
             VCVARS64_VS9_BAT_PATH = None
 
-        error1 = 'if errorlevel 1 {}'
+        error1 = 'IF %ERRORLEVEL% NEQ 0 {}'
 
         # Prefer VS9 proper over Microsoft Visual C++ Compiler for Python 2.7
         msvc_env_lines.append(build_vcvarsall_cmd(vcvarsall_vs_path))
@@ -195,17 +201,53 @@ def msvc_env_cmd(bits, config, override=None):
     return '\n'.join(msvc_env_lines) + '\n'
 
 
-def build(m, bld_bat, config):
-    with path_prepended(config.build_prefix):
-        env = environ.get_dict(config=config, m=m)
+def _write_bat_activation_text(file_handle, m):
+    file_handle.write('call "{conda_root}\\activate.bat" "{prefix}"\n'.format(
+        conda_root=root_script_dir,
+        prefix=m.config.build_prefix))
+    if m.is_cross:
+        # HACK: we need both build and host envs "active" - i.e. on PATH,
+        #     and with their activate.d scripts sourced. Conda only
+        #     lets us activate one, though. This is a
+        #     vile hack to trick conda into "stacking"
+        #     two environments.
+        #
+        # Net effect: binaries come from host first, then build
+        #
+        # Conda 4.4 may break this by reworking the activate scripts.
+        #  ^^ shouldn't be true
+        # In conda 4.4, export CONDA_MAX_SHLVL=2 to stack envs to two
+        #   levels deep.
+        # conda 4.4 does require that a conda-meta/history file
+        #   exists to identify a valid conda environment
+        history_file = join(m.config.host_prefix, 'conda-meta', 'history')
+        if not isfile(history_file):
+            if not isdir(dirname(history_file)):
+                os.makedirs(dirname(history_file))
+            open(history_file, 'a').close()
+        # removing this placeholder should make conda double-activate with conda 4.3
+        file_handle.write('set "PATH=%PATH:CONDA_PATH_PLACEHOLDER;=%"\n')
+        file_handle.write('set CONDA_MAX_SHLVL=2\n')
+        file_handle.write('call "{conda_root}\\activate.bat" "{prefix}"\n'.format(
+            conda_root=root_script_dir,
+            prefix=m.config.host_prefix))
+
+
+def build(m, bld_bat):
+    with path_prepended(m.config.build_prefix):
+        with path_prepended(m.config.host_prefix):
+            env = environ.get_dict(config=m.config, m=m)
     env["CONDA_BUILD_STATE"] = "BUILD"
 
+    # set variables like CONDA_PY in the test environment
+    env.update(set_language_env_vars(m.config.variant))
+
     for name in 'BIN', 'INC', 'LIB':
         path = env['LIBRARY_' + name]
         if not isdir(path):
             os.makedirs(path)
 
-    src_dir = config.work_dir
+    src_dir = m.config.work_dir
     if os.path.isfile(bld_bat):
         with open(bld_bat) as fi:
             data = fi.read()
@@ -215,20 +257,19 @@ def build(m, bld_bat, config):
             for key, value in env.items():
                 if value:
                     fo.write('set "{key}={value}"\n'.format(key=key, value=value))
-            fo.write(msvc_env_cmd(bits=bits, config=config,
-                                  override=m.get_value('build/msvc_compiler', None)))
+            if not m.uses_new_style_compiler_activation:
+                fo.write(msvc_env_cmd(bits=m.config.host_arch, config=m.config,
+                                    override=m.get_value('build/msvc_compiler', None)))
             # Reset echo on, because MSVC scripts might have turned it off
             fo.write('@echo on\n')
             fo.write('set "INCLUDE={};%INCLUDE%"\n'.format(env["LIBRARY_INC"]))
             fo.write('set "LIB={};%LIB%"\n'.format(env["LIBRARY_LIB"]))
-            if config.activate:
-                fo.write('call "{conda_root}\\activate.bat" "{prefix}"\n'.format(
-                    conda_root=root_script_dir,
-                    prefix=config.build_prefix))
+            if m.config.activate and m.name() != 'conda':
+                _write_bat_activation_text(fo, m)
             fo.write("REM ===== end generated header =====\n")
             fo.write(data)
 
         cmd = ['cmd.exe', '/c', 'bld.bat']
         check_call_env(cmd, cwd=src_dir)
 
-    fix_staged_scripts(join(config.build_prefix, 'Scripts'))
+    fix_staged_scripts(join(m.config.host_prefix, 'Scripts'), config=m.config)
